/*	Project sci88
	Serial com. interface PIC16F88
*/

#include <pic.h>

__CONFIG(
	  MCLRDIS
	& PWRTEN
	& BORDIS
	& UNPROTECT
	& WDTDIS
	& LVPDIS
	& INTIO
	& CCPRB3
	& DEBUGDIS
);

__CONFIG(
	  FCMDIS
	& IESODIS
);

__IDLOC(FF88);

#define SCI_PINIT() TRISB |= 0b00100100
#define SCI_SPBRG 51

#define XON 17
#define XOFF 19

unsigned char xf;
unsigned char rbuf[16];
unsigned char wp;
unsigned char rp;
unsigned char dc;

void interrupt entry(void){
	unsigned char c;

	if(RCIF){
		c = RCREG;

		if(c == XON || c == XOFF){
			xf = c;
			return;
		}
		if(dc == 16) return;
		rbuf[wp] = c;
		wp++;
		wp &= 15;
		dc++;
		if(dc == 12){
			while(TXIF == 0);
			TXREG = XOFF;
		}
	}
}

void sci_init(void){
	wp = 0;
	rp = 0;
	dc = 0;
	xf = XON;

	SCI_PINIT();

	SPBRG = SCI_SPBRG;
	TXSTA = 0b00100100;
	RCSTA = 0b10010000;
	RCIE = 1;
}

unsigned char getch(void){
	unsigned char c;

	while(dc == 0);
	RCIE = 0;
	c = rbuf[rp];
	rp++;
	rp &= 15;
	dc --;
	if(dc == 12){
		while(TXIF == 0);
		TXREG = XON;
	}
	RCIE = 1;
	return(c);
}

void putch(unsigned char c){
	while(xf == XOFF);
	while(TXIF == 0);
	TXREG = c;
}

void main(void){
	OSCCON = 0b01110000;
	OSCTUNE = 0;

	sci_init();
	PEIE = 1;
	ei();

	while(1)
		putch(getch());
}
